# Summary

## 前言

* [序言](README.md)

## 云原生

* [云原生的定义](cloud-native/cloud-native-definition.md)
* [CNCF - 云原生计算基金会简介](cloud-native/cncf.md)
* [CNCF章程](cloud-native/cncf-charter.md)
* [云原生的设计哲学](cloud-native/cloud-native-philosophy.md)
* [Play with Kubernetes](cloud-native/play-with-kubernetes.md)
* [快速部署一个云原生本地实验环境](cloud-native/cloud-native-local-quick-start.md)
* [Kubernetes与云原生应用概览](cloud-native/kubernetes-and-cloud-native-app-overview.md)
* [云原生应用之路——从Kubernetes到Cloud Native](cloud-native/from-kubernetes-to-cloud-native.md)
* [云原生编程语言](cloud-native/cloud-native-programming-languages.md)
  * [云原生编程语言Ballerina](cloud-native/cloud-native-programming-language-ballerina.md)
  * [云原生编程语言Pulumi](cloud-native/cloud-native-programming-language-pulumi.md)
* [云原生的未来](cloud-native/the-future-of-cloud-native.md)

## 概念与原理

* [Kubernetes架构](concepts/index.md)
  * [设计理念](concepts/concepts.md)
  * [Etcd解析](concepts/etcd.md)
  * [开放接口](concepts/open-interfaces.md)
    * [CRI - Container Runtime Interface（容器运行时接口）](concepts/cri.md)
    * [CNI - Container Network Interface（容器网络接口）](concepts/cni.md)
    * [CSI - Container Storage Interface（容器存储接口）](concepts/csi.md)
* [Kubernetes中的网络](concepts/networking.md)
  * [Kubernetes中的网络解析——以flannel为例](concepts/flannel.md)
  * [Kubernetes中的网络解析——以calico为例](concepts/calico.md)
  * [具备API感知的网络和安全性管理开源软件Cilium](concepts/cilium.md)
    * [Cilium架构设计与概念解析](concepts/cilium-concepts.md)
* [资源对象与基本概念解析](concepts/objects.md)
* [Pod状态与生命周期管理](concepts/pod-state-and-lifecycle.md)
  * [Pod概览](concepts/pod-overview.md)
  * [Pod解析](concepts/pod.md)
  * [Init容器](concepts/init-containers.md)
  * [Pause容器](concepts/pause-container.md)
  * [Pod安全策略](concepts/pod-security-policy.md)
  * [Pod的生命周期](concepts/pod-lifecycle.md)
  * [Pod Hook](concepts/pod-hook.md)
  * [Pod Preset](concepts/pod-preset.md)
  * [Pod中断与PDB（Pod中断预算）](concepts/pod-disruption-budget.md)
* [集群资源管理](concepts/cluster.md)
  * [Node](concepts/node.md)
  * [Namespace](concepts/namespace.md)
  * [Label](concepts/label.md)
  * [Annotation](concepts/annotation.md)
  * [Taint和Toleration（污点和容忍）](concepts/taint-and-toleration.md)
  * [垃圾收集](concepts/garbage-collection.md)
* [控制器](concepts/controllers.md)
  * [Deployment](concepts/deployment.md)
  * [StatefulSet](concepts/statefulset.md)
  * [DaemonSet](concepts/daemonset.md)
  * [ReplicationController和ReplicaSet](concepts/replicaset.md)
  * [Job](concepts/job.md)
  * [CronJob](concepts/cronjob.md)
  * [Horizontal Pod Autoscaling](concepts/horizontal-pod-autoscaling.md)
    * [自定义指标HPA](concepts/custom-metrics-hpa.md)
  * [准入控制器（Admission Controller）](concepts/admission-controller.md)
* [服务发现](concepts/service-discovery.md)
  * [Service](concepts/service.md)
  * [Ingress](concepts/ingress.md)
    * [Traefik Ingress Controller](concepts/traefik-ingress-controller.md)
* [身份与权限控制](concepts/authentication-and-permission.md)
  * [ServiceAccount](concepts/serviceaccount.md)
  * [RBAC——基于角色的访问控制](concepts/rbac.md)
  * [NetworkPolicy](concepts/network-policy.md)
* [存储](concepts/storage.md)
  * [Secret](concepts/secret.md)
  * [ConfigMap](concepts/configmap.md)
    * [ConfigMap的热更新](concepts/configmap-hot-update.md)
  * [Volume](concepts/volume.md)
  * [Persistent Volume（持久化卷）](concepts/persistent-volume.md)
  * [Storage Class](concepts/storageclass.md)
  * [本地持久化存储](concepts/local-persistent-storage.md)
* [集群扩展](concepts/extension.md)
  * [使用自定义资源扩展API](concepts/custom-resource.md)
  * [使用CRD扩展Kubernetes API](concepts/crd.md)
  * [Aggregated API Server](concepts/aggregated-api-server.md)
  * [APIService](concepts/apiservice.md)
  * [Service Catalog](concepts/service-catalog.md)
* [资源调度](concepts/scheduling.md)
  * [QoS（服务质量等级）](concepts/qos.md)

## 用户指南

* [用户指南](guide/index.md)
* [资源对象配置](guide/resource-configuration.md)
  * [配置Pod的liveness和readiness探针](guide/configure-liveness-readiness-probes.md)
  * [配置Pod的Service Account](guide/configure-pod-service-account.md)
  * [Secret配置](guide/secret-configuration.md)
  * [管理namespace中的资源配额](guide/resource-quota-management.md)
* [命令使用](guide/command-usage.md)
  * [Docker用户过度到kubectl命令行指南](guide/docker-cli-to-kubectl.md)
  * [kubectl命令概览](guide/using-kubectl.md)
  * [kubectl命令技巧大全](guide/kubectl-cheatsheet.md)
  * [使用etcdctl访问kubernetes数据](guide/using-etcdctl-to-access-kubernetes-data.md)
* [集群安全性管理](guide/cluster-security-management.md)
  * [管理集群中的TLS](guide/managing-tls-in-a-cluster.md)
  * [kubelet的认证授权](guide/kubelet-authentication-authorization.md)
  * [TLS bootstrap](guide/tls-bootstrapping.md)
  * [创建用户认证授权的kubeconfig文件](guide/kubectl-user-authentication-authorization.md)
  * [IP伪装代理](guide/ip-masq-agent.md)
  * [使用kubeconfig或token进行用户身份认证](guide/auth-with-kubeconfig-or-token.md)
  * [Kubernetes中的用户与身份认证授权](guide/authentication.md)
  * [Kubernetes集群安全性配置最佳实践](guide/kubernetes-security-best-practice.md)
* [访问Kubernetes集群](guide/access-kubernetes-cluster.md)
  * [访问集群](guide/access-cluster.md)
  * [使用kubeconfig文件配置跨集群认证](guide/authenticate-across-clusters-kubeconfig.md)
  * [通过端口转发访问集群中的应用程序](guide/connecting-to-applications-port-forward.md)
  * [使用service访问群集中的应用程序](guide/service-access-application-cluster.md)
  * [从外部访问Kubernetes中的Pod](guide/accessing-kubernetes-pods-from-outside-of-the-cluster.md)
  * [Cabin - Kubernetes手机客户端](guide/cabin-mobile-dashboard-for-kubernetes.md)
  * [Kubernetic - Kubernetes桌面客户端](guide/kubernetes-desktop-client.md)
  * [Kubernator - 更底层的Kubernetes UI](guide/kubernator-kubernetes-ui.md)
* [在Kubernetes中开发部署应用](guide/application-development-deployment-flow.md)
  * [适用于kubernetes的应用开发部署流程](guide/deploy-applications-in-kubernetes.md)
  * [迁移传统应用到Kubernetes中——以Hadoop YARN为例](guide/migrating-hadoop-yarn-to-kubernetes.md)
  * [使用StatefulSet部署用状态应用](guide/using-statefulset.md)

## 最佳实践

* [最佳实践概览](practice/index.md)
* [在CentOS上部署Kubernetes集群](practice/install-kubernetes-on-centos.md)
  * [创建TLS证书和秘钥](practice/create-tls-and-secret-key.md)
  * [创建kubeconfig文件](practice/create-kubeconfig.md)
  * [创建高可用etcd集群](practice/etcd-cluster-installation.md)
  * [安装kubectl命令行工具](practice/kubectl-installation.md)
  * [部署master节点](practice/master-installation.md)
  * [安装flannel网络插件](practice/flannel-installation.md)
  * [部署node节点](practice/node-installation.md)
  * [安装kubedns插件](practice/kubedns-addon-installation.md)
  * [安装dashboard插件](practice/dashboard-addon-installation.md)
  * [安装heapster插件](practice/heapster-addon-installation.md)
  * [安装EFK插件](practice/efk-addon-installation.md)
* [生产级的Kubernetes简化管理工具kubeadm](practice/install-kubernetes-with-kubeadm.md)
  * [使用kubeadm在Ubuntu Server 16.04上快速构建测试集群](practice/install-kubernetes-on-ubuntu-server-16.04-with-kubeadm.md)
* [服务发现与负载均衡](practice/service-discovery-and-loadbalancing.md)
  * [安装Traefik ingress](practice/traefik-ingress-installation.md)
  * [分布式负载测试](practice/distributed-load-test.md)
  * [网络和集群性能测试](practice/network-and-cluster-perfermance-test.md)
  * [边缘节点配置](practice/edge-node-configuration.md)
  * [安装Nginx ingress](practice/nginx-ingress-installation.md)
  * [安装配置DNS](practice/dns-installation.md)
    * [安装配置Kube-dns](practice/configuring-dns.md)
    * [安装配置CoreDNS](practice/coredns.md)
* [运维管理](practice/operation.md)
  * [Master节点高可用](practice/master-ha.md)
  * [服务滚动升级](practice/service-rolling-update.md)
  * [应用日志收集](practice/app-log-collection.md)
  * [配置最佳实践](practice/configuration-best-practice.md)
  * [集群及应用监控](practice/monitor.md)
  * [数据持久化问题](practice/data-persistence-problem.md)
  * [管理容器的计算资源](practice/manage-compute-resources-container.md)
  * [集群联邦](practice/federation.md)
* [存储管理](practice/storage.md)
  * [GlusterFS](practice/glusterfs.md)
    * [使用GlusterFS做持久化存储](practice/using-glusterfs-for-persistent-storage.md)
    * [使用Heketi作为Kubernetes的持久存储GlusterFS的external provisioner](practice/using-heketi-gluster-for-persistent-storage.md)
    * [在OpenShift中使用GlusterFS做持久化存储](practice/storage-for-containers-using-glusterfs-with-openshift.md)
  * [GlusterD-2.0](practice/glusterd-2.0.md)
  * [Ceph](practice/ceph.md)
    * [用Helm托管安装Ceph集群并提供后端存储](practice/ceph-helm-install-guide-zh.md)
    * [使用Ceph做持久化存储](practice/using-ceph-for-persistent-storage.md)
    * [使用rbd-provisioner提供rbd持久化存储](practice/rbd-provisioner.md)
  * [OpenEBS](practice/openebs.md)
    * [使用OpenEBS做持久化存储](practice/using-openebs-for-persistent-storage.md)
  * [Rook](practice/rook.md)
  * [NFS](practice/nfs.md)
    * [利用NFS动态提供Kubernetes后端存储卷](practice/using-nfs-for-persistent-storage.md)
* [集群与应用监控](practice/monitoring.md)
  * [Heapster](practice//heapster.md)
    * [使用Heapster获取集群和对象的metric数据](practice/using-heapster-to-get-object-metrics.md)
  * [Prometheus](practice/prometheus.md)
    * [使用Prometheus监控kubernetes集群](practice/using-prometheus-to-monitor-kuberentes-cluster.md)
    * [Prometheus查询语言PromQL使用说明](practice/promql.md)
  * [使用Vistio监控Istio服务网格中的流量](practice/vistio-visualize-your-istio-mesh.md)
* [分布式跟踪](practice/distributed-tracing.md)
  * [OpenTracing](practice/opentracing.md)
* [服务编排管理](practice/services-management-tool.md)
  * [使用Helm管理Kubernetes应用](practice/helm.md)
  * [构建私有Chart仓库](practice/create-private-charts-repo.md)
* [持续集成与发布](practice/ci-cd.md)
  * [使用Jenkins进行持续集成与发布](practice/jenkins-ci-cd.md)
  * [使用Drone进行持续集成与发布](practice/drone-ci-cd.md)
* [更新与升级](practice/update-and-upgrade.md)
  * [手动升级Kubernetes集群](practice/manually-upgrade.md)
  * [升级dashboard](practice/dashboard-upgrade.md)

## 领域应用

* [领域应用概览](usecases/index.md)
* [微服务架构](usecases/microservices.md)
  * [微服务中的服务发现](usecases/service-discovery-in-microservices.md)
  * [使用Java构建微服务并发布到Kubernetes平台](usecases/microservices-for-java-developers.md)
    * [Spring Boot快速开始指南](usecases/spring-boot-quick-start-guide.md)
* [Service Mesh 服务网格](usecases/service-mesh.md)
  * [企业级服务网格架构](usecases/the-enterprise-path-to-service-mesh-architectures.md)
    * [Service Mesh基础](usecases/service-mesh-fundamental.md)
    * [Service Mesh技术对比](usecases/comparing-service-mesh-technologies.md)
    * [采纳和演进](usecases/service-mesh-adoption-and-evolution.md)
    * [定制和集成](usecases/service-mesh-customization-and-integration.md)
    * [总结](usecases/service-mesh-conclusion.md)
  * [Istio](usecases/istio.md)
    * [安装并试用Istio service mesh](usecases/istio-installation.md)
    * [配置请求的路由规则](usecases/configuring-request-routing.md)
    * [安装和拓展Istio service mesh](usecases/install-and-expand-istio-mesh.md)
    * [集成虚拟机](usecases/integrating-vms.md)
    * [Istio中sidecar的注入规范及示例](usecases/sidecar-spec-in-istio.md)
    * [如何参与Istio社区及注意事项](usecases/istio-community-tips.md)
    * [Istio教程](usecases/istio-tutorial.md)
    * [Istio免费学习资源汇总](usecases/istio-tutorials-collection.md)
    * [深入理解Istio中的Sidecar注入与流量劫持](usecases/understand-sidecar-injection-and-traffic-hijack-in-istio-service-mesh.md)
  * [Linkerd](usecases/linkerd.md)
    * [Linkerd 使用指南](usecases/linkerd-user-guide.md)
  * [Conduit](usecases/conduit.md)
    * [Condiut概览](usecases/conduit-overview.md)
    * [安装Conduit](usecases/conduit-installation.md)
  * [Envoy](usecases/envoy.md)
    * [Envoy的架构与基本术语](usecases/envoy-terminology.md)
    * [Envoy作为前端代理](usecases/envoy-front-proxy.md)
    * [Envoy mesh教程](usecases/envoy-mesh-in-kubernetes-tutorial.md)
  * [SOFAMesh](usecases/sofamesh.md)
    * [SOFAMesh中的Dubbo on x-protocol](usecases/dubbo-on-x-protocol-in-sofa-mesh.md)
  * [SOFAMosn](usecases/sofamosn.md)
    * [使用 SOFAMosn 构建 SOFAMesh](usecases/sofamosn-in-sofamesh.md)
* [大数据](usecases/big-data.md)
  * [Spark standalone on Kubernetes](usecases/spark-standalone-on-kubernetes.md)
  * [运行支持Kubernetes原生调度的Spark程序](usecases/running-spark-with-kubernetes-native-scheduler.md)
* [Serverless架构](usecases/serverless.md)
  * [理解Serverless](usecases/understanding-serverless.md)
  * [FaaS-函数即服务](usecases/faas.md)
    * [OpenFaaS快速入门指南](usecases/openfaas-quick-start.md)
* [边缘计算](usecases/edge-computing.md)
* [人工智能](usecases/ai.md)

## 开发指南

* [开发指南概览](develop/index.md)
* [SIG和工作组](develop/sigs-and-working-group.md)
* [开发环境搭建](develop/developing-environment.md)
  * [本地分布式开发环境搭建（使用Vagrant和Virtualbox）](develop/using-vagrant-and-virtualbox-for-development.md)
* [单元测试和集成测试](develop/testing.md)
* [client-go示例](develop/client-go-sample.md)
* [Operator](develop/operator.md)
  * [operator-sdk](develop/operator-sdk.md)
* [kubebuilder](develop/kubebuilder.md)
* [高级开发指南](develop/advance-developer.md)
* [社区贡献](develop/contribute.md)
* [Minikube](develop/minikube.md)

## 附录

* [附录说明](appendix/index.md)
* [Kubernetes中的应用故障排查](appendix/debug-kubernetes-services.md)
* [Kubernetes相关资讯和情报链接](appendix/material-share.md)
* [Docker最佳实践](appendix/docker-best-practice.md)
* [使用技巧](appendix/tricks.md)
* [问题记录](appendix/issues.md)
* [Kubernetes版本更新日志](appendix/kubernetes-changelog.md)
  * [Kubernetes1.7更新日志](appendix/kubernetes-1.7-changelog.md)
  * [Kubernetes1.8更新日志](appendix/kubernetes-1.8-changelog.md)
  * [Kubernetes1.9更新日志](appendix/kubernetes-1.9-changelog.md)
  * [Kubernetes1.10更新日志](appendix/kubernetes-1.10-changelog.md)
  * [Kubernetes1.11更新日志](appendix/kubernetes-1.11-changelog.md)
  * [Kubernetes1.12更新日志](appendix/kubernetes-1.12-changelog.md)
  * [Kubernetes1.13更新日志](appendix/kubernetes-1.13-changelog.md)
* [Kubernetes及云原生年度总结及展望](appendix/summary-and-outlook.md)
  * [Kubernetes与云原生2017年年终总结及2018年展望](appendix/kubernetes-and-cloud-native-summary-in-2017-and-outlook-for-2018.md)
* [Kubernetes认证服务提供商（KCSP）说明](appendix/about-kcsp.md)
* [认证Kubernetes管理员（CKA）说明](appendix/about-cka-candidate.md)
